/*
'astring' by Dintiradan - Version 1.0
Last updated December 30, 2006
Contact me with any suggestions or problems at dintiradan@yahoo.ca

Feel free to use and modify this program for your scenario, provided:
	Original documentation is preserved (unless it no longer applies),
	Documentation exists for new features, and
	Contact information is maintained (for all contributors to the program).
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define STR_LEN 254

void precheck (void);
void postcheck (void);

int main (void) {
	char file_name[STR_LEN];
	FILE * directory;
	int astring_present = 0;
	
	/* All files in directory in 'astringdir.txt', separated by newlines */
	system("dir /A-d-h-s-r /B >astringdir.txt");
	
	if (! (directory = fopen("astringdir.txt", "r") ) ) {
		fprintf(stderr, "Cannot open 'astringdir.txt'\n");
		exit(EXIT_FAILURE);
	}
	
	for (;;) {
		if (! fgets(file_name, STR_LEN, directory) ) {
			if (feof(directory) )
				break;
			else {
				fprintf(stderr,
						"Cannot read from 'astringdir.txt'\n");
				exit(EXIT_FAILURE);
			}
		}
		
		if (! strcmp(file_name, "astring.txt\n") ) {
			astring_present = 1;
			break;
		}
	}
	
	if (fclose(directory) )
		fprintf(stderr, "Warning: Cannot close 'astringdir.txt'\n");
	
	if (astring_present)
		postcheck();
	else
		precheck();
	
	if (remove("astringdir.txt") )
		fprintf(stderr,
				"Warning: Cannot remove 'astringdir.txt'\n");
	
	return 0;
}

void precheck (void) {
	FILE * astring, * script, * directory;
	char script_name[STR_LEN], input_char;
	int is_in_string, string_length, line;
	
	if (! (astring = fopen("astring.txt", "w") ) ) {
		fprintf(stderr, "Cannot open 'astring.txt'\n");
		return;
	}
	
	if (! (directory = fopen("astringdir.txt", "r") ) ) {
		fprintf(stderr, "Cannot open 'astringdir.txt'\n");
		return;
	}
	
	for (;;) { /* For each file */
		if (! fgets(script_name, STR_LEN, directory) ) {
			if (feof(directory) )
				break;
			else {
				fprintf(stderr, "Cannot read from 'astringdir.txt'\n");
				return;
			}
		}
		
		script_name[strlen(script_name) - 1] = 0; /* Chop */
		
		if (strcmp(script_name + strlen(script_name) - 4, ".txt") )
			continue;
		
		if (! strncmp(script_name, "astring", 7) )
			continue;
		
		if (! (script = fopen(script_name, "r") ) ) {
			fprintf(stderr, "Warning: Cannot open '%s'\n", script_name);
			continue;
		}
		
		fputs("// ", astring);
		fputs(script_name, astring);
		fputc('\n', astring);
		is_in_string = 0;
		string_length = 0;
		line = 1;
		
		for (;;) {
			if ( (input_char = fgetc(script) ) == EOF) {
				if (feof(script) )
					break;
				else {
					fprintf(stderr, "Cannot read from '%s' (line %d)\n",
							script_name, line);
					return;
				}
			}
			
			if (is_in_string) {
				if (input_char == '"') {
					is_in_string = 0;
					fputc('\n', astring);
					continue;
				}
				else if (input_char == '\n') { /* Only test for Windows */
					fprintf(stderr, "Warning: Unterminated string in '%s' "
									"(line %d)\n", script_name, line++);
					is_in_string = 0;
					fputc('\n', astring);
					continue;
				}
				else if (input_char == '_')
					input_char = '"';
				
				if (++string_length <= STR_LEN)
					fputc(input_char, astring);
				else if (string_length == STR_LEN + 1) /* Warn once */
					fprintf(stderr, "Warning: Truncated string from '%s' "
									"(line %d)\n", script_name, line);
			}
			else {
				if (input_char == '"') {
					is_in_string = 1;
					string_length = 0;
				}
				else if (input_char == '\n') /* Only test for Windows */
					line++;
			}
		}
		
		if (fclose(script) )
			fprintf(stderr, "Warning: Cannot close '%s'\n", script_name);
	}
	
	if (fclose(astring) )
		fprintf(stderr, "Warning: Cannot close 'astring.txt'\n");
	
	if (fclose(directory) )
		fprintf(stderr, "Warning: Cannot close 'astringdir.txt'\n");
}

void postcheck (void) {
	FILE * astring, * script, * temp_script;
	char file_line[STR_LEN], script_name[STR_LEN], input_char;
	int is_in_string, i, line, unterm_string;
	
	if (! (astring = fopen("astring.txt", "r") ) ) {
		fprintf(stderr, "Cannot open 'astring.txt'\n");
		return;
	}
	
	for (;;) { /* For each file */
		if (! fgets(file_line, STR_LEN, astring) ) {
			if (feof(astring) )
				break;
			else {
				fprintf(stderr, "Cannot read filename from 'astring.txt'\n");
				return;
			}
		}
		
		if (! sscanf(file_line, "// %[^\n]", script_name) ) {
			fprintf(stderr, "Expected filename, got '%s'\n", file_line);
			return;
		}
		
		if (! (script = fopen(script_name, "r") ) ) {
			fprintf(stderr, "Cannot open '%s'\n", script_name);
			return;
		}
		
		is_in_string = 0;
		unterm_string = 0;
		line = 1;
		
		if (! (temp_script = fopen("astringtmp.txt", "w") ) ) {
			fprintf(stderr, "Cannot open 'astringtmp.txt' (Script %s)\n",
					script_name);
			return;
		}
		
		for (;;) { /* For each string */
			while (! is_in_string) {
				if ( (input_char = fgetc(script) ) == EOF) {
					if (feof(script) )
						break;
					else {
						fprintf(stderr, "Cannot read from '%s' (line %d)\n",
								script_name, line);
						return;
					}
				}
				
				if (input_char == '\n') /* Only test for Windows */
					line++;
				else if (input_char == '"')
					is_in_string = 1;
				
				fputc(input_char, temp_script);
			}
			
			if (feof(script) )
				break;
			
			for (i = 0; i < STR_LEN; i++) { /* Insert string */
				if ( (input_char = fgetc(astring) ) == EOF) {
					if (feof(astring) )
						fprintf(stderr, "Expected string not found in "
										"'astring.txt' for '%s' (line %d)\n",
										script_name, line);
					else
						fprintf(stderr, "Cannot read from 'astring.txt'\n");
					
					return;
				}
				
				if (input_char == '\n')
					break;
				
				if (input_char == '"')
					input_char = '_';
				
				fputc(input_char, temp_script);
			}
			
			if (i == STR_LEN && fgetc(astring) != '\n') {
				fprintf(stderr, "Warning: Truncated string placed in '%s' "
								"(line %d)\n", script_name, line);
				
				/* Advance astring to next line */
				while ( (input_char = fgetc(astring) ) != '\n') {
					if (input_char == EOF) {
						fprintf(stderr, "Cannot advance 'astring.txt'\n");
						return;
					}
				}
			}
			
			/* Advance script past endquote */
			while ( (input_char = fgetc(script) ) != '"') {
				if (input_char == EOF) {
					if (feof(script) ) {
						fprintf(stderr, "Warning: Fixed unterminated string "
										"in '%s' (line %d)\n", script_name,
										line);
						unterm_string = 1;
						break;
					}
					else {
						fprintf(stderr, "Cannot advance character in '%s' "
										"(line %d)\n", script_name, line);
						return;
					}
				}
				
				if (input_char == '\n') { /* Only test for Windows */
					fprintf(stderr, "Warning: Fixed unterminated string in "
									"'%s' (line %d)\n", script_name, line++);
					unterm_string = 1;
					break;
				}
			}
			
			fputc('"', temp_script);
			is_in_string = 0;
			
			if (unterm_string) {
				fputc('\n', temp_script);
				unterm_string = 0;
			}
		}
		
		if (fclose(temp_script) )
			fprintf(stderr, "Warning: Cannot close 'astringtmp.txt' (Script "
							"%s)\n", script_name);
		
		if (fclose(script) )
			fprintf(stderr, "Warning: Cannot close '%s'\n", script_name);
		
		if (remove(script_name) )
			fprintf(stderr, "Warning: Cannot remove '%s'\n", script_name);
		
		if (rename("astringtmp.txt", script_name) )
			fprintf(stderr, "Warning: Cannot rename 'astringtmp.txt' to "
							"'%s'\n", script_name);
	}
	
	if (fclose(astring) )
		fprintf(stderr, "Warning: Cannot close 'astring.txt'\n");
	
	if (remove("astring.txt") )
		fprintf(stderr, "Warning: Cannot remove 'astring.txt'\n");
}
